@@ -77,15 +77,13 @@
 #include <gobject/gvaluecollector.h>
 #include <string.h>
 
-GST_DEBUG_CATEGORY_STATIC (gst_tag_interface_debug);
-#define GST_CAT_DEFAULT tag_tag_interface_debug
-
 static GQuark gst_tag_key;
 
 typedef struct
 {
   GstTagMergeMode mode;
   GstTagList *list;
+  GStaticMutex lock;
 } GstTagData;
 
 GType
@@ -107,9 +105,6 @@ gst_tag_setter_get_type (void)
       NULL
     };
 
-    GST_DEBUG_CATEGORY_INIT (gst_tag_interface_debug, "GstTagInterface", 0,
-        "interfaces for tagging");
-
     _type = g_type_register_static (G_TYPE_INTERFACE, "GstTagSetter",
         &tag_setter_info, 0);
 
@@ -130,6 +125,8 @@ gst_tag_data_free (gpointer p)
   if (data->list)
     gst_tag_list_free (data->list);
 
+  g_static_mutex_free (&data->lock);
+
   g_slice_free (GstTagData, data);
 }
 
@@ -140,11 +137,20 @@ gst_tag_setter_get_data (GstTagSetter * setter)
 
   data = g_object_get_qdata (G_OBJECT (setter), gst_tag_key);
   if (!data) {
-    data = g_slice_new (GstTagData);
-    data->list = NULL;
-    data->mode = GST_TAG_MERGE_KEEP;
-    g_object_set_qdata_full (G_OBJECT (setter), gst_tag_key, data,
-        gst_tag_data_free);
+    static GStaticMutex create_mutex = G_STATIC_MUTEX_INIT;
+
+    /* make sure no other thread is creating a GstTagData at the same time */
+    g_static_mutex_lock (&create_mutex);
+    data = g_object_get_qdata (G_OBJECT (setter), gst_tag_key);
+    if (!data) {
+      data = g_slice_new (GstTagData);
+      g_static_mutex_init (&data->lock);
+      data->list = NULL;
+      data->mode = GST_TAG_MERGE_KEEP;
+      g_object_set_qdata_full (G_OBJECT (setter), gst_tag_key, data,
+          gst_tag_data_free);
+    }
+    g_static_mutex_unlock (&create_mutex);
   }
 
   return data;
@@ -167,10 +173,13 @@ gst_tag_setter_reset_tags (GstTagSetter * setter)
   g_return_if_fail (GST_IS_TAG_SETTER (setter));
 
   data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
   if (data->list) {
     gst_tag_list_free (data->list);
     data->list = NULL;
   }
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -192,12 +201,15 @@ gst_tag_setter_merge_tags (GstTagSetter * setter, const GstTagList * list,
   g_return_if_fail (GST_IS_TAG_LIST (list));
 
   data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
   if (data->list == NULL) {
     if (mode != GST_TAG_MERGE_KEEP_ALL)
       data->list = gst_tag_list_copy (list);
   } else {
     gst_tag_list_insert (data->list, list, mode);
   }
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -268,10 +280,14 @@ gst_tag_setter_add_tag_valist (GstTagSetter * setter, GstTagMergeMode mode,
   g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
 
   data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
   if (!data->list)
     data->list = gst_tag_list_new ();
 
   gst_tag_list_add_valist (data->list, mode, tag, var_args);
+
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -294,10 +310,15 @@ gst_tag_setter_add_tag_valist_values (GstTagSetter * setter,
   g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
 
   data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
+
   if (!data->list)
     data->list = gst_tag_list_new ();
 
   gst_tag_list_add_valist_values (data->list, mode, tag, var_args);
+
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -321,10 +342,15 @@ gst_tag_setter_add_tag_value (GstTagSetter * setter,
   g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
 
   data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
+
   if (!data->list)
     data->list = gst_tag_list_new ();
 
   gst_tag_list_add_value (data->list, mode, tag, value);
+
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -334,8 +360,10 @@ gst_tag_setter_add_tag_value (GstTagSetter * setter,
  * Returns the current list of tags the setter uses.  The list should not be
  * modified or freed.
  *
- * Returns: a current snapshot of the taglist used in the setter
- *          or NULL if none is used.
+ * This function is not thread-safe.
+ *
+ * Returns: (transfer none): a current snapshot of the taglist used in the
+ *          setter or NULL if none is used.
  */
 G_CONST_RETURN GstTagList *
 gst_tag_setter_get_tag_list (GstTagSetter * setter)
@@ -357,10 +385,16 @@ gst_tag_setter_get_tag_list (GstTagSetter * setter)
 void
 gst_tag_setter_set_tag_merge_mode (GstTagSetter * setter, GstTagMergeMode mode)
 {
+  GstTagData *data;
+
   g_return_if_fail (GST_IS_TAG_SETTER (setter));
   g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
 
-  gst_tag_setter_get_data (setter)->mode = mode;
+  data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
+  data->mode = mode;
+  g_static_mutex_unlock (&data->lock);
 }
 
 /**
@@ -375,7 +409,16 @@ gst_tag_setter_set_tag_merge_mode (GstTagSetter * setter, GstTagMergeMode mode)
 GstTagMergeMode
 gst_tag_setter_get_tag_merge_mode (GstTagSetter * setter)
 {
+  GstTagMergeMode mode;
+  GstTagData *data;
+
   g_return_val_if_fail (GST_IS_TAG_SETTER (setter), FALSE);
 
-  return gst_tag_setter_get_data (setter)->mode;
+  data = gst_tag_setter_get_data (setter);
+
+  g_static_mutex_lock (&data->lock);
+  mode = data->mode;
+  g_static_mutex_unlock (&data->lock);
+
+  return mode;
 }
